@@ -5,39 +5,59 @@ import mimetypes
 import os
 
 from qtpy import QtGui
+from qtpy import QtWidgets
 
 from . import qtcompat
 from . import resources
 from .compat import ustr
 from .decorators import memoize
+from .i18n import N_
 
 
 KNOWN_FILE_MIME_TYPES = [
-    ('text',    'file-code.svg'),
-    ('image',   'file-media.svg'),
-    ('python',  'file-code.svg'),
-    ('ruby',    'file-code.svg'),
-    ('shell',   'file-code.svg'),
-    ('perl',    'file-code.svg'),
-    ('octet',   'file-binary.svg'),
+    ('text', 'file-code.svg'),
+    ('image', 'file-media.svg'),
+    ('octet', 'file-binary.svg'),
 ]
 
 KNOWN_FILE_EXTENSIONS = {
-    '.java':    'file-code.svg',
-    '.groovy':  'file-code.svg',
-    '.cpp':     'file-code.svg',
-    '.c':       'file-code.svg',
-    '.h':       'file-code.svg',
-    '.cxx':     'file-code.svg',
+    '.bash': 'file-code.svg',
+    '.c': 'file-code.svg',
+    '.cpp': 'file-code.svg',
+    '.css': 'file-code.svg',
+    '.cxx': 'file-code.svg',
+    '.h': 'file-code.svg',
+    '.hpp': 'file-code.svg',
+    '.hs': 'file-code.svg',
+    '.html': 'file-code.svg',
+    '.java': 'file-code.svg',
+    '.js': 'file-code.svg',
+    '.ksh': 'file-code.svg',
+    '.lisp': 'file-code.svg',
+    '.perl': 'file-code.svg',
+    '.pl': 'file-code.svg',
+    '.py': 'file-code.svg',
+    '.rb': 'file-code.svg',
+    '.rs': 'file-code.svg',
+    '.sh': 'file-code.svg',
+    '.zsh': 'file-code.svg',
 }
 
 
-def install(icon_themes):
-    for theme in icon_themes:
+def install(themes):
+    for theme in themes:
         icon_dir = resources.icon_dir(theme)
         qtcompat.add_search_path('icons', icon_dir)
 
 
+def icon_themes():
+    return (
+        (N_('Default'), 'default'),
+        (N_('Dark Theme'), 'dark'),
+        (N_('Light Theme'), 'light'),
+    )
+
+
 def name_from_basename(basename):
     """Prefix the basename with "icons:" so that git-cola's icons are found
 
@@ -66,7 +86,7 @@ def from_theme(name, fallback=None):
 
     """
     if hasattr(QtGui.QIcon, 'fromTheme'):
-        base, ext = os.path.splitext(name)
+        base, _ = os.path.splitext(name)
         if fallback:
             qicon = QtGui.QIcon.fromTheme(base, icon(fallback))
         else:
@@ -93,25 +113,24 @@ def from_filename(filename):
     return from_name(name_from_basename(basename))
 
 
-def mkicon(icon, default=None):
-    if icon is None and default is not None:
-        icon = default()
-    elif icon and isinstance(icon, (str, ustr)):
-        icon = QtGui.QIcon(icon)
-    return icon
+def mkicon(value, default=None):
+    if value is None and default is not None:
+        value = default()
+    elif value and isinstance(value, (str, ustr)):
+        value = QtGui.QIcon(value)
+    return value
 
 
-@memoize
 def from_style(key):
     """Maintain a cache of standard icons and return cache entries."""
-    style = QtGui.QApplication.instance().style()
+    style = QtWidgets.QApplication.instance().style()
     return style.standardIcon(key)
 
 
-def status(filename, deleted, staged, untracked):
+def status(filename, deleted, is_staged, untracked):
     if deleted:
         icon_name = 'circle-slash-red.svg'
-    elif staged:
+    elif is_staged:
         icon_name = 'staged.svg'
     elif untracked:
         icon_name = 'question-plain.svg'
@@ -126,6 +145,10 @@ def add():
     return from_theme('list-add', fallback='plus.svg')
 
 
+def alphabetical():
+    return icon('a-z-order.svg')
+
+
 def branch():
     return icon('git-branch.svg')
 
@@ -143,7 +166,7 @@ def cola():
 
 
 def commit():
-    return icon('git-commit.svg')
+    return icon('document-save-symbolic.svg')
 
 
 def compare():
@@ -222,6 +245,10 @@ def merge():
     return icon('git-merge.svg')
 
 
+def modified():
+    return icon('modified.svg')
+
+
 def modified_name():
     return name_from_basename('modified.svg')
 
@@ -262,6 +289,10 @@ def repo():
     return icon('repo.svg')
 
 
+def reverse_chronological():
+    return icon('last-first-order.svg')
+
+
 def save():
     return icon('desktop-download.svg')
 
