@@ -7,6 +7,7 @@ import os
 
 from .. import core
 from .. import gitcmds
+from .. import version
 from ..git import STDOUT
 from ..observable import Observable
 from . import prefs
@@ -32,6 +33,7 @@ class MainModel(Observable):
     message_images_changed = 'images_changed'
     message_mode_about_to_change = 'mode_about_to_change'
     message_mode_changed = 'mode_changed'
+    message_ref_sort_changed = 'ref_sort_changed'
     message_submodules_changed = 'message_submodules_changed'
     message_updated = 'updated'
 
@@ -96,6 +98,7 @@ class MainModel(Observable):
         self.submodules = set()
         self.submodules_list = []
 
+        self.ref_sort = 0  # (0: version, 1:reverse-chrono)
         self.local_branches = []
         self.remote_branches = []
         self.tags = []
@@ -282,8 +285,13 @@ class MainModel(Observable):
 
     def _update_branches_and_tags(self):
         context = self.context
+        sort_types = (
+            'version:refname',
+            '-committerdate',
+        )
+        sort_key = sort_types[self.ref_sort]
         local_branches, remote_branches, tags = gitcmds.all_refs(
-            context, split=True)
+            context, split=True, sort_key=sort_key)
         self.local_branches = local_branches
         self.remote_branches = remote_branches
         self.tags = tags
@@ -344,16 +352,18 @@ class MainModel(Observable):
         return gitcmds.remote_url(self.context, name, push=push)
 
     def fetch(self, remote, **opts):
-        return run_remote_action(self.git.fetch, remote, **opts)
+        return run_remote_action(self.context, self.git.fetch, remote, **opts)
 
     def push(self, remote, remote_branch='', local_branch='', **opts):
         # Swap the branches in push mode (reverse of fetch)
         opts.update(dict(local_branch=remote_branch,
                          remote_branch=local_branch))
-        return run_remote_action(self.git.push, remote, push=True, **opts)
+        return run_remote_action(self.context, self.git.push, remote,
+                                 push=True, **opts)
 
     def pull(self, remote, **opts):
-        return run_remote_action(self.git.pull, remote, pull=True, **opts)
+        return run_remote_action(self.context, self.git.pull, remote,
+                                 pull=True, **opts)
 
     def create_branch(self, name, base, track=False, force=False):
         """Create a branch named 'name' from revision 'base'
@@ -393,9 +403,22 @@ class MainModel(Observable):
             return self.directory
         return core.getcwd()
 
+    def cycle_ref_sort(self):
+        """Choose the next ref sort type (version, reverse-chronological)"""
+        self.set_ref_sort(self.ref_sort + 1)
+
+    def set_ref_sort(self, raw_value):
+        value = raw_value % 2  # Currently two sort types
+        if value == self.ref_sort:
+            return
+        self.ref_sort = value
+        self._update_branches_and_tags()
+        self.notify_observers(self.message_ref_sort_changed)
+
 
 # Helpers
-def remote_args(remote,
+# pylint: disable=too-many-arguments
+def remote_args(context, remote,
                 local_branch='',
                 remote_branch='',
                 ff_only=False,
@@ -425,7 +448,11 @@ def remote_args(remote,
         elif no_ff:
             kwargs['no_ff'] = True
     elif force:
-        kwargs['force'] = True
+        # pylint: disable=simplifiable-if-statement
+        if push and version.check_git(context, 'force-with-lease'):
+            kwargs['force_with_lease'] = True
+        else:
+            kwargs['force'] = True
 
     if push and set_upstream:
         kwargs['set_upstream'] = True
@@ -454,6 +481,6 @@ def refspec_arg(local_branch, remote_branch, pull, push):
     return what
 
 
-def run_remote_action(action, remote, **kwargs):
-    args, kwargs = remote_args(remote, **kwargs)
+def run_remote_action(context, action, remote, **kwargs):
+    args, kwargs = remote_args(context, remote, **kwargs)
     return action(*args, **kwargs)
